<?php

namespace Drupal\http_headers_cleaner\Service;

/**
 * Clean meta.
 *
 * Please see /admin/config/system/http-headers-cleaner.
 *
 * @package Drupal\http_headers_cleaner\Service
 */
class MetaCleaner {

  /**
   * Nom du service.
   *
   * @const string
   */
  const SERVICE_NAME = 'http_headers_cleaner.meta_cleaner';

  /**
   * Settings.
   *
   * @var \Drupal\http_headers_cleaner\Service\HttpHeadersCleanerSettings
   */
  protected $settings;

  /**
   * The list of configured patterns.
   *
   * @var array|mixed
   */
  protected $patterns;

  /**
   * MetaCleaner constructor.
   *
   * @param \Drupal\http_headers_cleaner\Service\HttpHeadersCleanerSettings $settings
   *   The settings.
   */
  public function __construct(HttpHeadersCleanerSettings $settings) {
    $this->settings = $settings;
    $this->patterns = $this->settings->getConfig(HttpHeadersCleanerSettings::TYPE_METATAGS);
  }

  /**
   * Retourne le singleton.
   *
   * @return static
   *   Le singleton.
   */
  public static function me() {
    return \Drupal::service(static::SERVICE_NAME);
  }

  /**
   * Alter meta attachments.
   *
   * @param array $attachments
   *   The attachments array.
   */
  public function alterAttachments(array &$attachments) {
    if ($this->settings->isEnabled(HttpHeadersCleanerSettings::TYPE_METATAGS)) {

      // Process html_head.
      if (isset($attachments['#attached']['html_head'])) {
        $this->processHtmlHead($attachments['#attached']['html_head']);
      }

      // Process html_head_link.
      if (isset($attachments['#attached']['html_head_link'])) {
        $this->processHtmlHeadLink($attachments['#attached']['html_head_link']);
      }
    }
  }

  /**
   * Clean Head.
   *
   * @param array $htmlHead
   *   The head tags value list.
   */
  protected function processHtmlHead(array &$htmlHead) {
    for ($key = count($htmlHead); $key--; $key > -1) {
      if ($typePatterns = $this->getTypePatternFromMeta($htmlHead[$key][0])) {
        if (isset($htmlHead[$key][0]['#attributes'])) {
          if ($this->matchesAttributes($htmlHead[$key][0]['#attributes'], $typePatterns)) {
            unset($htmlHead[$key]);
          }
        }
      }
    }
  }

  /**
   * Clean head links.
   *
   * @param array $htmlHeadLink
   *   The head links list.
   */
  protected function processHtmlHeadLink(array &$htmlHeadLink) {
    if (isset($this->patterns['link']) && $typePatterns = $this->patterns['link']) {
      if (!empty($typePatterns)) {
        for ($key = count($htmlHeadLink); $key--; $key > -1) {
          foreach ($htmlHeadLink[$key] as $data) {
            // Only attributes.
            if (is_array($data) && $this->matchesAttributes($data, $typePatterns)) {
              unset($htmlHeadLink[$key]);
            }
          }
        }
      }
    }
  }

  /**
   * Return true if meta matches with pattern.
   *
   * @param array $metaData
   *   The metat data.
   *
   * @return string
   *   The type.
   */
  protected function getTypePatternFromMeta(array $metaData) {
    if (isset($metaData['#tag']) && array_key_exists($metaData['#tag'], $this->patterns)) {
      return $this->patterns[$metaData['#tag']];
    }
  }

  /**
   * Check if attributes matches type patterns.
   *
   * @param array $attributes
   *   Attributes.
   * @param string $typePatterns
   *   Type pattern.
   *
   * @return bool
   *   Match status.
   */
  protected function matchesAttributes(array $attributes, $typePatterns) {
    if (!empty($typePatterns)) {
      // Check attributes.
      foreach ($attributes as $key => $value) {
        // Check attributes key.
        if (array_key_exists($key, $typePatterns)) {
          foreach ($typePatterns[$key] as $pattern) {
            // Check attributes value.
            if (preg_match($pattern, $value)) {
              return TRUE;
            }
          }
        }
      }
    }

    return FALSE;
  }

}
